@@ -44,7 +44,12 @@ impl Graph {
     fn _greedy_next_node(&self, current: Node, candidates: &Vec<Node>) -> Node {
         let mut best_edge_dist = f32::INFINITY;
         let mut best_neighbour: Node = *candidates.last().unwrap();
-        for &i in candidates {
+        let candidates = match candidates.len() == 0 {
+            true => self._find_neighbours(current),
+            false => candidates.clone()
+        };
+
+        for i in candidates {
             if i != current {
                 let edge_dist = self.distance_matrix[current][i];
                 if edge_dist <= best_edge_dist {
@@ -56,6 +61,17 @@ impl Graph {
         best_neighbour
     }
 
+    fn _find_neighbours(&self, current: Node) -> Vec<Node> {
+        let mut candidates: Vec<Node> = vec![];
+        let n_nodes = self.distance_matrix[current].len();
+        for i in sample(&mut thread_rng(), n_nodes, n_nodes) {
+            if self.distance_matrix[current][i] != 0. {
+                candidates.push(i);
+            }
+        }
+        candidates
+    }
+
     fn evaluate_disjoint_values(&self, paths: &Vec<Vec<Node>>) -> Vec<f32> {
         let mut disj_values = vec![];
         let mut passage_matrix: Vec<Vec<Vec<bool>>> = vec![vec![vec![false; self.distance_matrix.len()];
@@ -185,7 +201,6 @@ impl Graph {
 
     fn find_candidate_nodes(&self, current_node: Node, cl_size: usize, visited: &Vec<Node>) -> Vec<Node> {
         let mut candidates = vec![];
-
         let n_nodes = self.distance_matrix[current_node].len();
         for i in sample(&mut thread_rng(), n_nodes, n_nodes) {
             if self.distance_matrix[current_node][i] != 0. && !visited.contains(&i) {
@@ -454,7 +469,7 @@ impl MacsOptimizer {
 #[pyfunction]
 fn optimize_macs(distance_matrix: Vec<Vec<f32>>, n_ants: usize, n_types: usize,
                  init_pheromones: f32, beta: f32, q0: f32, gamma: f32, rho: f32, source: usize,
-                 dest: usize, t_max: usize, cl_len: usize) -> PyResult<Vec<(Vec<Node>, f32, f32)>> {
+                 dest: usize, t_max: usize, cl_len: usize, ) -> PyResult<Vec<(Vec<Node>, f32, f32)>> {
     let mut macs = MacsOptimizer::new(n_ants, n_types, init_pheromones, beta, q0, gamma,
         rho, &distance_matrix);
     let best_paths = macs.optimize(source, dest, t_max, cl_len);
